@@ -1,5 +1,5 @@
 package Object::Depot;
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 use strictures 2;
 
 =encoding utf8
@@ -202,8 +202,8 @@ has class => (
 =head2 constructor
 
     constuctor => sub{
-        my ($depot, $args) = @_;
-        return $depot->class->new( $args );
+        my ($args) = @_;
+        return __PACKAGE__->depot->class->new( $args );
     },
 
 Set this to a code ref to control how objects get constructed.
@@ -224,17 +224,19 @@ has constructor => (
     isa => CodeRef,
 );
 
-my $class_constructor = sub{
-    my $depot = shift;
-    return $depot->class->new( @_ );
-};
-
 my $undef_constructor = sub{ undef };
 
 sub _build_constructor {
     my ($self) = @_;
-    return $class_constructor if $self->_has_class();
-    return $undef_constructor;
+
+    return $undef_constructor if !$self->_has_class();
+
+    return _build_class_constructor( $self->class() );
+}
+
+sub _build_class_constructor {
+    my ($class) = @_;
+    return sub{ $class->new( @_ ) };
 }
 
 =head2 type
@@ -429,7 +431,8 @@ sub fetch {
 sub _fetch {
     my ($self, $key) = @_;
 
-    my $object = $self->_objects->{$key};
+    my $object = $self->_injections->{ $key };
+    $object ||= $self->_objects->{$key};
     return $object if $object;
 
     return undef if !$self->_has_class();
@@ -522,7 +525,7 @@ sub _create {
 
     my $args = $self->_arguments( $key, $extra_args );
 
-    my $object = $self->constructor->( $self, $args );
+    my $object = $self->constructor->( $args );
 
     croakf(
         'Constructor returned an invalid value, %s, for key %s: %s',
